@@ -10,12 +10,12 @@ class GtkNull(object):
 		@note Thread agnostic
 		"""
 		self._projectsCombo = widgetTree.get_widget("projectsCombo")
-		self._addTaskButton = widgetTree.get_widget("add-addTaskButton")
+		self._addTaskButton = widgetTree.get_widget("add-addButton")
 
 		self._todoBox = widgetTree.get_widget("todoBox")
 		self._todoAreaFiller = gtk.Label()
 
-		self._manager = null_backend.NullManager("", "")
+		self._manager = null_backend.NullBackend("", "")
 
 	@staticmethod
 	def name():
